// Arduino Mega-2560: AC-Spannungs-Logger von Peter Klingbeil (c)2020/21 Steuersoftware
// Ein Nachbau fuer private Zwecke ist erlaubt.
//
// Benötigt Arduino UNO Sensor Software zur Spannungsmessung: AC_Voltage_Measuring.ino
// Spannungsmessung galvanisch getrennt (AC 0-1000V), Bedienung mit 5 Taster 
// Ausgabe mit 2 stelligen I2C 16 Zeichen/Zeile Display
// Dieses Programm nutzt diverse Public Domain Elemente (siehe #include)
// Und benoetigt einen Arduino Mega 2560 als Steuerelement und einen Arduino UNO
// fuer den Sensor zur Berechnung des Messwertes vom Sensorelemnt ZMPT101B
// Benoetigt werden ausserdem:
// Eine Realtime Uhr DS3231 mit Batterie
// Ein SPI Reader Micro Speicherkartenmodul mit SD-Karte
// Ein DC 9-12 V Netzteil (nicht getaktet)Stromabgabe ca.120 mA
// Version 2.0 mit neuen Funktionen und mehr Stabilitaet

#include <Wire.h>
#include <LCD.h>
#include <LiquidCrystal_I2C.h>
#include <String.h>
#include "RTClib.h" 

// RT-Clock Initialisierung

RTC_DS3231 rtc;
bool syncOnFirstStart = false; // true, falls die Zeitinformationen der RTC mit dem PC synchronisiert werden sollen.
                               // sollte standardmäßig auf false stehen   

// Display Initialisierung
LiquidCrystal_I2C lcd(0x3F,2,1,0,4,5,6,7,3,POSITIVE);
  
// SD-Card Initialisierung
// SD card attached to SPI bus as follows:
// ** MOSI - pin 11 on Arduino Uno
// ** MISO - pin 12 on Arduino Uno
// ** CLK  - pin 13 on Arduino Uno
// ** CS   - pin 10 on Arduino Uno

// ** MOSI - pin 51 on Arduino Mega 2560
// ** MISO - pin 50 on Arduino Mega 2560
// ** CLK  - pin 52 on Arduino Mega 2560
// ** CS   - pin 53 on Arduino Mega 2560

#define __AVR_ATmega2560__
#include <SPI.h>
#include <SD.h>

// set up variables using the SD utility library functions:
  
  Sd2Card card;
  SdVolume volume;
  SdFile root;
  const int chipSelect = 53;
  
// Globale Variablen
   
  String Funktion = String("Uhr                "); // Anzeige der Funktionsauswahl oben
  String Funktion2 = String("                ");   // Anzeige bei Run/Stop
  String Werte =    String("00:00:00           "); // Wert Anzeige Zeile unten
  String Stunde = String("12");                    // Zeit
  String Minute = String("30");                    // Zeit
  String Sekunde = String("30");                   // Zeit
  String X = String("  ");                         // Zeit
  String Jahr = String("2021");                    // Datum
  String Monat = String("06");
  String Tag = String("15");

  uint8_t _Stunde = 12;                  // Zeit Stell- Variable
  uint8_t _Minute = 30;                  // Zeit
  uint8_t _Sekunde = 30;                 // Zeit
  uint16_t _Jahr = 2020;                 // Datum
  uint8_t _Monat = 6;
  uint8_t _Tag = 15;

  String TimeArray[6];   // Zeit
  String NameArray[6];   // Zeitnamen
    
  volatile unsigned int FunktionCounter = 1; // 1=Uhr 2=Uhr stellen 3=Trigger 4=Start/Stop 5=Speicher 6 Messwerte  7 Daten Aufzeichnung
  const byte interruptPin = 2;           // Funktionstaste auf Digital Digital Input Pin 2
  volatile int InterruptDone = 0;        // Interrupt Flag für die Funktionstaste
  volatile unsigned long alteZeit=0, entprellZeit=500;  // Taste entprellen 400 ms -NEW-
  volatile int FC=0, FC1=0;              // Auswahl Counter
  int Value;                             // Konvertierung String to int
  char BUF[6];                           // dito.
  int Mess=0;                            // Vergleich? Spannung=0, Frequenz=1
  int DeltaU = 5;                        // Delta Überspannung in %
  int DeltaU2 = 2;                       // Delta Unterspannung in %
  String Bufx=String("00");              // zum Konvertieren von char
  int Trigg=0;                           // Trigger Auswahl 0=Spannung 1=Frequenz

  long Sensor = 0;                       // Sensor analog input, here it's A0
  long Sensor2 = 0;                      // Berechnung der AC Spannung
  volatile int SensorDelta = 0;          // Korrektur Wert
  int HundertP = Sensor;                 // Vergleichsvariable = 100%
  int Ueber = 0;                         // Schwellwerte
  int Unter = 0;
  int SetTrigger = 0;                    // Schwellwert erreicht = 1
  int SetTrigger2 = 0;                   // Flag
  int Counter = 0;                       // Anzahl der Trigger
  float current_Volts = 0;               // Messwert vom Sensor

  long SlaveReceived = 0;                // I2C Bus
  String data = "       ";               // I2C Daten vom Arduino Sensor
  boolean y=true;                        // Tasten entprellen
  int Zyklus=1000;                       // Abfrage Zyklus 1,5,20 Sekunden
  int Set = 0;                           // Logger Zeitinterval
  boolean FLAG = true;                   // Allgemeines Flag
  boolean ALARM = false;                 // Alarm Unter oder Ueberspannung
  boolean Debug = false;                 // Monitor Test Ausgaben
  
void setup() 
{
  Wire.begin();                // initiate I2C Bus
  
  if (Debug == true)
  {
  Serial.begin(9600);          // Debuging
  while (!Serial);             // wait for serial monitor
  }

  pinMode(interruptPin, INPUT_PULLUP); // Pin 2 = Taste A Funktionsauswahl
  pinMode(4, INPUT_PULLUP);   // Taste 1 Auswahl
  pinMode(5, INPUT_PULLUP);   // Taste 2 auf
  pinMode(6, INPUT_PULLUP);   // Taste 3 ab
  pinMode(7, INPUT_PULLUP);   // Taste 4 Setzen
  pinMode(8, OUTPUT);         // AC-Sensor selektions Pin
  pinMode(9, OUTPUT);         // Alarmausgang (z.B. LED)
  
  Wire.begin(8);                          //Begins I2C communication with Slave Address as 8 at pin (A4,A5)
  Wire.onReceive(receiveEvent);           //Function call when Slave receives value from master I2C
  Wire.onRequest(requestEvent);           //Function call when Master request value from Slave I2C

  attachInterrupt(digitalPinToInterrupt(interruptPin), FTASTE, CHANGE);  // Funktionstaste A
  
  lcd.begin(16, 2);            // Display Initialisierung
  lcd.setCursor(0,0);
  lcd.print("Spannungs Logger");  // Versionsanzeige
  lcd.setCursor(0,1);
  lcd.print("  Version 2.0   ");
  delay(1000);                 // Versionsanzeige fuer 1 Sekunde

  if (Debug == true) Serial.println("Board Startet - Software Version 2.0");  // Debuging

  if (rtc.lostPower())         // Test der Uhr
  {
    lcd.setCursor(0,1);
    lcd.print("RTC lost power !");
    delay(2000);

    // following line sets the RTC to the date & time this sketch was compiled
    // rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }
  else
  {
    if (Debug == true) Serial.println("Uhr ist OK"); 
  }
 
  lcd.setCursor(0,1);          // Test der SD Speicherkarte

 if (!SD.begin(chipSelect)) 
  {
    lcd.print("SD-Karte Fehler!");
   } else 
  {
    lcd.print("SD-Karte ist OK.");
    if (Debug == true) Serial.println("SD-Karte ist OK");
  } 

  delay(2000);
  lcd.setCursor(0,1);          // Anzeige neu initialisieren
  lcd.print("Weiter > Taste A");                       
  lcd.setCursor(0,0);
  FunktionCounter=0;
}

void receiveEvent (int howMany)   //This Function is called when Slave receives value from master
{  
  data = "";
  while( Wire.available())
  {
  data += (char)Wire.read();
  }  
  SlaveReceived = Wire.read();    //Used to read value received from master and store in variable SlaveReceived
  //Serial.println(data);
}

void requestEvent()   // This Function is called when Master wants value from slave
{
  Wire.write(0);      // sends one byte to master
}

void PRINT(int x)   // Display Ausgabe mit/ohne LCD loeschen
{
    if (x==1) lcd.clear();
    lcd.setCursor(0,0);
    lcd.print(Funktion);  // Anzeige 1. Zeile
    lcd.setCursor(0,1);
    lcd.print(Werte);     // Anzeige 2. Zeile      
}

void FTASTE()   // Interrupt & Entprellung der Funktions taste A
{
   if((millis() - alteZeit) > entprellZeit) 
   { 
    // innerhalb der entprellZeit nichts machen
    InterruptDone=1;
    ++FunktionCounter;
    if (FunktionCounter > 7) 
     { 
      FunktionCounter=1; 
     }
    alteZeit = millis(); // letzte Schaltzeit merken      
  }  
}

int UpDown(int Set)  // Up-Down Taste um Werte zu stellen
 {

  if(digitalRead(5) == LOW and y == true) //Auf-Taste betätigt
   {
    y=false;
    delay(250);
    ++Set;
    y=true;
    return Set;
   }
  if (digitalRead(6) == LOW and y == true) //Ab-Taste betätigt
   {
    y=false;
    delay(250);
    --Set;
    y=true;
    return Set;
   }
 }


void UHR()   // Uhr Anzeigen
{
  InterruptDone=0;
  while (FunktionCounter == 1)
  {
  // Aktuelle Zeit übernehmen 
  
  DateTime now = rtc.now();    
  Stunde=now.hour();
  Minute=now.minute();
  Sekunde=now.second();
  Jahr=now.year();
  Monat=now.month();
  Tag=now.day();
  
  // Mit Null auffüllen fals notwendig (2 Stellen)
  
  X=Stunde;
  if (Stunde.length()== 1) { Stunde = "0" + X; }

  X=Minute;
  if (Minute.length()== 1) { Minute = "0" + X; }

  X=Sekunde;
  if (Sekunde.length()== 1) { Sekunde = "0" + X; }

  X=Tag;
  if (Tag.length()== 1) { Tag = "0" + X; }

  X=Monat;
  if (Monat.length()== 1) { Monat = "0" + X; }
  
  Funktion = "Zeit  " + Tag + "." + Monat + "." + Jahr + " ";
  Werte= "    " + Stunde + ":" + Minute + ":" + Sekunde + "    ";

  PRINT(0);
  delay(500);
  }
}

void GetTime()   // Uhrzeit laden
 {
   DateTime now = rtc.now();
    
   // Aktuelle Zeit übernehmen 
   
   Stunde=now.hour();
   Minute=now.minute();
   Sekunde=now.second();
   Jahr=now.year();
   Monat=now.month();
   Tag=now.day();

    // Null Stellen Korrektur
    X=Stunde;
    if (Stunde.length()== 1) { Stunde = "0" + X; }
    X=Minute;
    if (Minute.length()== 1) { Minute = "0" + X; }
    X=Sekunde;
    if (Sekunde.length()== 1) { Sekunde = "0" + X; }
    X=Tag;
    if (Tag.length()== 1) { Tag = "0" + X; }
    X=Monat;
    if (Monat.length()== 1) { Monat = "0" + X; }
   
   TimeArray[0] = now.day();
   TimeArray[1] = now.month();
   TimeArray[2] = now.year();
   TimeArray[3] = now.hour();
   TimeArray[4] = now.minute();
   TimeArray[5] = now.second();

   NameArray[0] = "Tag";
   NameArray[1] = "Monat";
   NameArray[2] = "Jahr";
   NameArray[3] = "Stunde";
   NameArray[4] = "Minute";
   NameArray[5] = "Sekunde";   
 }

void UHRSET()   // Uhr stellen
{
   lcd.clear();
   FC=0;
   FC1=0;
   InterruptDone=0;
   
   GetTime();
 
   Funktion="  Uhr stellen  ";
   Werte=   "    Taste 1    ";
   PRINT(0);
  
   while (FunktionCounter == 2)
   {
     if ((digitalRead(4) == LOW) and y==true) //Wahltaste 1 betätigt
     {
      y=false;
      delay(250);
      if (FC > 5) FC=0;   // Arrays 0 bis 5
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print(">1  +2  -3 Set 4");
      lcd.setCursor(0,1);
      lcd.print(NameArray[FC]);
      lcd.print(" = ");
      lcd.print(TimeArray[FC]);
      FC1=FC;
      ++FC;
      y=true;       
     }

     delay(200);
     if ((digitalRead(5) == LOW) or (digitalRead(6)) == LOW) 
     {     
      delay(300);
      
      Value=TimeArray[FC1].toInt();
      if (digitalRead(5) == LOW) ++Value;   //Auf-Taste betätigt
      if (digitalRead(6) == LOW) --Value;  //Ab-Taste betätigt
            
      // Eingabe: Grenzwerte prüfen     
      
      if(FC1==0)   // Eingabe Korrekturen Tag
      {
        if (Value > 31) Value=31;
        if (Value < 1) Value=1;
      }

       if(FC1==1)   // Eingabe Korrekturen Monat
      {
        if (Value > 12) Value=12;
        if (Value < 1) Value=1;
      }
      
      if(FC1==2)   // Eingabe Korrekturen Jahr
      {
        if (Value < 2020) Value=2020;
      }
      
      if(FC1==3)   // Eingabe Korrekturen Stunde
      {
        if (Value > 23) Value=23;
        if (Value < 1) Value=1;
      }
      
      if(FC1==4)   // Eingabe Korrekturen Minute
      {
        if (Value > 59) Value=59;
        if (Value < 1) Value=1;
      }

      if(FC1==5)   // Eingabe Korrekturen Sekunde
      {
        if (Value > 59) Value=59;
        if (Value < 1) Value=1;
      }
      
      itoa(Value,BUF,10);
      TimeArray[FC1]=BUF;
      
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print(NameArray[FC1]);
      lcd.print(" weiter 1");
      lcd.setCursor(0,1);
      lcd.print(TimeArray[FC1]);

     }
          
    if (digitalRead(7) == LOW)  // Taste 4 setzt Uhrzeit und Datum
     {
      delay(300);

      // Zeit übernehmen 
   
      _Tag=TimeArray[0].toInt();
      _Monat=TimeArray[1].toInt();
      _Jahr=TimeArray[2].toInt();
      _Stunde=TimeArray[3].toInt();
      _Minute=TimeArray[4].toInt();
      _Sekunde=TimeArray[5].toInt();

      if (Debug == true) Serial.println(_Sekunde);
      
      rtc.adjust(DateTime(_Jahr, _Monat, _Tag, _Stunde, _Minute, _Sekunde));
      // rtc.adjust(DateTime(2020, 02, 14, 18, 14, 00)); RTC Zeit setzen
      
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("   Weiter mit   ");
      lcd.setCursor(0,1);
      lcd.print("    Taste A     ");
      }
     delay(100);    
    } 
}


void TRIGGER()   // Trigger Werte einstellen (Ueber- Unterspannung)
{
    InterruptDone=0;
    lcd.clear();
    Mess=0;
    
    while (FunktionCounter == 3)
    {  
    Funktion="DeltaU > Taste 1";
    
    if (Mess == 0)
     {
      itoa(DeltaU,BUF,10);
      Bufx=BUF;
      Werte="U+ = " + Bufx + " % ";
     }
    else
     {
      itoa(DeltaU2,BUF,10);
      Bufx=BUF;
      Werte="U- = " + Bufx + " % ";
     }
     
    PRINT(0);
    delay(250);
    
    if (digitalRead(4) == LOW)
     {
      if (Mess==0) Mess=1;
      else Mess=0;
     }
     if ((digitalRead(5) == LOW) or (digitalRead(6)) == LOW)  // % Werte einstellen
     {
      if (Mess==0) DeltaU=UpDown(DeltaU);
      if (Mess==1) DeltaU2=UpDown(DeltaU2);
     
      if (DeltaU > 25) DeltaU=25;  // Eingabe Korrektur Grenzwerte
      if (DeltaU < 1) DeltaU=1;
      if (DeltaU2 > 25) DeltaU2=25;
      if (DeltaU2 < 1) DeltaU2=1;    
     }
   }
}

int Messung()   // Eingangsspannung anzeigen zum justieren
{
  InterruptDone=0;
  SensorDelta=0;   // Delta auf 0 Volt setzen
  Funktion=String("  -- Justage -- ");
  Werte=String("    warten...   ");
  PRINT(0);
  delay(1000);
   
  digitalWrite(8, LOW);  // AC-Sensor = einschalten
  delay(200);
  Sensor = data.toInt(); // read the analog in value. Der Wert ist mit 100 multipliziert
  if (Debug == true) Serial.println(Sensor);
  Sensor = Sensor + 50;  // Aufrunden
  Sensor = Sensor/100;   // auf 3 Stellen runden
  HundertP=Sensor;       // Referenzwert normalisieren
  digitalWrite(8, HIGH);  // AC-Sensor = ausschalten
  
  delay(200);
  
  while (FunktionCounter == 6)
   {
    if(digitalRead(5) == LOW and y == true) //Auf-Taste betätigt
    {
    y=false;
    delay(250);
    SensorDelta=SensorDelta+1;
    y=true;
   }
   
   if (digitalRead(6) == LOW and y == true) //Ab-Taste betätigt
   {
    y=false;
    delay(250);
    SensorDelta=SensorDelta-1;
    y=true;
   }
  
   //lcd.clear();
   delay(200);    
   lcd.setCursor(0,0);
   lcd.print("Set & Ende A +/-");
   lcd.setCursor(0,1);
   lcd.print("Korrektur: ");
   lcd.print(Sensor + SensorDelta);
   lcd.print(" V");
  }
}

void RUNSTOP()  // Grenzwert Messung
{
  InterruptDone=0;
  Funktion=String(" Start Messung  ");
  Werte=String("    Taste 1     ");
  PRINT(0);
  SetTrigger2=0;
  
 while ((SetTrigger2 == 0) and (FunktionCounter == 4)) // Messung starten
  {
  if (digitalRead(4) == LOW and y==true)  // Start Taste 1
   {
    y=false;
    digitalWrite(8, LOW);  // AC-Sensor = einschalten
    delay(200);

    // Grenzwerte vorbelegen
    Sensor = data.toInt(); // read the analog in value
    if (Debug == true) Serial.println(SensorDelta);
    Sensor = Sensor + 50;  // Aufrunden
    Sensor = Sensor/100;   // auf 3 Stellen runden
    Sensor = Sensor + SensorDelta;
    HundertP=Sensor;       // Referenzwert speichern
    digitalWrite(8, HIGH);  // AC-Sensor = ausschalten
    delay(200);
    
    // Grenzwert Berechnung: HundertP/100=1Prozent +/- X
    // Hier wird der aktuell gelesene Wert und nicht die reale Spannung berechnet
    Ueber=((HundertP/100)*DeltaU) + HundertP;    
    Unter=HundertP-((HundertP/100)*DeltaU2);

    if (Debug == true)
    {
    Serial.print("Ueber ");
    Serial.print(Ueber);
    Serial.print(" - Unter ");
    Serial.println(Unter);
    }

    GetTime();
    Funktion = String(Tag + "." + Monat + ". ;" + Stunde + ":" + Minute + "." + Sekunde);
    Funktion2 = String(Tag + "." + Monat + ". " + Stunde + ":" + Minute + "." + Sekunde);

    Counter=0;   // Zähler der Trigger Ereignisse

    File dataFile = SD.open("datalog.txt", FILE_WRITE);  // SD-Karte
    // if the file is available, write to it:
    if (dataFile) 
    {
     dataFile.print("RUN > ;");
     dataFile.print(Sensor);
     dataFile.print(" ;");
     dataFile.println(Funktion);
     dataFile.close();
     if (Debug == true)
     {
      Serial.print(" Start Messung ");
      Serial.println(Funktion2);
     }
    }
    y=true;
    SetTrigger2=1;  // while Schleife verlassen
  }
 }   
   lcd.clear();     
   lcd.setCursor(0,0);
   lcd.print("Startwert: ");
   lcd.print(Sensor);
   lcd.print(" V");
 
  SetTrigger=1;
  digitalWrite(8, LOW);  // AC-Sensor = einschalten
  while (FunktionCounter == 4)  // Messzyklus Schleife. Abbruch durch Interupt mit Taste A
  {  
    digitalWrite(8, LOW);  // AC-Sensor = einschalten

    if (ALARM == true)  // Ereignis dauerhaft anzeigen
     {
      digitalWrite(9, HIGH);  // LED an
     }
     
     //else
     //{
     // digitalWrite(9, LOW);  // LED aus
     //}
    
    GetTime();
    Funktion = String(Tag + "." + Monat + ". ;" + Stunde + ":" + Minute + "." + Sekunde);
    Funktion2 = String(Tag + "." + Monat + ". " + Stunde + ":" + Minute + "." + Sekunde);
  
    Sensor = data.toInt();  // read the analog in value
    if (Debug == true) Serial.println(SensorDelta);
    Sensor2=Sensor;
    Sensor = Sensor + 50;   // Aufrunden 0,5 Volt
    Sensor = Sensor/100;    // auf 3 Stellen runden
    Sensor = Sensor + SensorDelta;
    HundertP=Sensor;       // Referenzwert jetzt neu speichern

    // Pause zwischen der Messwert- Erfassung Messzyklus ist 50 ms
    delay(50);
    
    // Grenzwert Berechnung: HundertP/100=1Prozent +/- X
    // Hier wird der aktuell gelesene Wert und nicht die reale Spannung berechnet
    
    //Ueber=((HundertP/100)*DeltaU) + HundertP;    
    //Unter=HundertP-((HundertP/100)*DeltaU2);
         
    if (Debug == true)
    {
    Serial.print("AC=");
    Serial.print(Sensor);
    Serial.print(" IN=");
    Serial.print(Sensor2);
    Serial.print(" Trigger ");
    Serial.print(SetTrigger);
    Serial.print("  Ueber ");
    Serial.print(Ueber);
    Serial.print("  Unter ");
    Serial.println(Unter);
    }
     
   if ((Sensor > Unter) and (Sensor < Ueber) and (SetTrigger == 0))  // Spannung im Normbereich?
   {
     SetTrigger=1;
     GetTime();
     delay(300);
     Funktion = String(Tag + "." + Monat + ". ;" + Stunde + ":" + Minute + "." + Sekunde);
     Funktion2 = String(Tag + "." + Monat + ". " + Stunde + ":" + Minute + "." + Sekunde);
     File dataFile = SD.open("datalog.txt", FILE_WRITE);  // SD-Karte
     dataFile.print("OK  > ;");
     dataFile.print(Sensor);
     dataFile.print(" ;");
     dataFile.println(Funktion);
     dataFile.close();
     if (Debug == true) Serial.println("Normal ");
     ALARM=false;
     lcd.clear();
     lcd.setCursor(0,0);
     lcd.print("Normal ");
     lcd.print(Sensor);
     lcd.print(" ");
     lcd.print(Counter);
     lcd.setCursor(0,1);
     lcd.print(Funktion2);
   }
   
  if ((Sensor < Unter) and (SetTrigger == 1))  // Unterspannung
   {
    GetTime();
    SetTrigger=0;
    ++Counter;
    Funktion = String(Tag + "." + Monat + ". ;" + Stunde + ":" + Minute + "." + Sekunde);
    Funktion2 = String(Tag + "." + Monat + ". " + Stunde + ":" + Minute + "." + Sekunde);
    File dataFile = SD.open("datalog.txt", FILE_WRITE);  // SD-Karte
    dataFile.print("U-  > ;");
    dataFile.print(Sensor);
    dataFile.print(" ;");
    dataFile.println(Funktion);
    dataFile.close();
    if (Debug == true) Serial.println("U-     ");
    ALARM=true;
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Trigg. -U ");
    lcd.print(Sensor);
    lcd.print(" ");
    lcd.print(Counter);
    lcd.setCursor(0,1);
    lcd.print(Funktion2);
   }
   
  if ((Sensor > Ueber) and (SetTrigger == 1))  // Ueberspannung
   {
    GetTime();
    SetTrigger=0;
    ++Counter;
    delay(300);
    Funktion = String(Tag + "." + Monat + ". ;" + Stunde + ":" + Minute + "." + Sekunde);
    Funktion2 = String(Tag + "." + Monat + ". " + Stunde + ":" + Minute + "." + Sekunde);
    File dataFile = SD.open("datalog.txt", FILE_WRITE);  // SD-Karte
    dataFile.print("U+  > ;");
    dataFile.print(Sensor);
    dataFile.print(" ;");
    dataFile.println(Funktion);
    dataFile.close();
    if (Debug == true) Serial.print("U+     ");
    ALARM=true;
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Trigg. +U ");
    lcd.print(Sensor);
    lcd.print(" ");
    lcd.print(Counter);
    lcd.setCursor(0,1);
    lcd.print(Funktion2);
   }
 } 
}

void Logger()
{
 int Set = 0;
 y=true;
 InterruptDone=0;
 Funktion="    Logging     ";
 Werte=" Start=1 Zykl=2 ";
 PRINT(0);
 delay(300);
 GetTime();
 Funktion = String("LG" + Tag + Stunde + Minute + ".csv");  // Variabler Dateiname
 if (Debug == true) Serial.println(Funktion);

 while (FunktionCounter == 7)
  {
    
   if (digitalRead(4) == LOW) // Logger Starten
   {   
    File dataFile2 = SD.open(Funktion, FILE_WRITE); // SD-Karte Log-Datei EXCEL
    // if the file is available, write to it:
    if (dataFile2)
    {
    while(InterruptDone==0)
     {
     digitalWrite(8, LOW); // AC-Sensor = einschalten
     delay(100);
     Sensor = data.toInt(); // read the analog in value
     Sensor = Sensor + (SensorDelta * 100);
     //Sensor = Sensor + 50; // Aufrunden
     //Sensor = Sensor/100; // auf 3 Stellen runden
     HundertP=Sensor;       // Referenzwert normalisieren
     if (Debug == true) Serial.println(HundertP);
     digitalWrite(8, HIGH); // AC-Sensor = ausschalten um I2C Bus Kollisionen zu vermeiden!
     
     GetTime();
     
     Funktion = String(Tag + "." + Monat + ";" + Stunde + ":" + Minute + "." + Sekunde);
     Werte = String("    " + Stunde + ":" + Minute + ":" + Sekunde + "    ");
     // Aktuellen Wert anzeigen

     delay(50);
     lcd.setCursor(0,0);
     lcd.print("Logging: ");
     lcd.print(Sensor);
     lcd.println(" V   ");
     lcd.setCursor(0,1);
     lcd.println(Werte);
     digitalWrite(8, LOW); // AC-Sensor = einschalten
     dataFile2.print(Funktion);
     dataFile2.print(";");
     dataFile2.println(HundertP);

     delay(Zyklus); // Pause zwischen der Messwert- Erfassung
     }
   }
  dataFile2.close();
 }
 
    if ((digitalRead(5) == LOW) and (y==true))   // Zykluszeit 1, 5, 20 Sekunden
     {
      y=false;
      Funktion="   Zykluszeit   ";
      Werte=   "Set=4 weiter > A";
      PRINT(0);      
      
      while(InterruptDone==0)
      {        
       if (digitalRead(7) == LOW)
       {
        delay(400);
        lcd.clear();
        if (Set > 2) Set=0;
        if (Set == 0) Zyklus=1000;
        if (Set == 1) Zyklus=5000;
        if (Set == 2) Zyklus=20000;
        Set=Set+1;
        lcd.setCursor(0,0);
        lcd.print("Set=4 weiter > A");
        lcd.setCursor(0,1);
        lcd.print(Zyklus);
        lcd.print(" MSekunden ");
      }
     }
   }
  }
}

  
void SPEICHER()  // Speicher Funktionen
{  
 String buf;    // Read from file
 int lf=10;     // Linefeed Zeichen
 int X=59;      // Semikolon ist das Trennzeichen für EXCEL

   InterruptDone=0;
   y=true;
   Funktion="  SD-Speicher   ";
   Werte=   " Lesen=2  Loe=3 ";
   PRINT(0);
   delay(100);
   
  while (FunktionCounter == 5)
  {  
     if (digitalRead(5) == LOW and y==true)   // lesen mit Taste 2
      {
       y=false;
       delay(250);
       Werte=   "Auslesen mit T3";
       PRINT(0);
       delay(400);
       
       File dataFile = SD.open("datalog.txt", FILE_READ);

       if (dataFile) 
       {
        while (dataFile.available()) 
        {
         delay(400);
         if (digitalRead(6) == LOW)   // lesen Taste 3
         {
         lcd.clear();
         buf=dataFile.readStringUntil(X);
         if (Debug == true) Serial.print(buf);
         lcd.setCursor(0,0);
         lcd.print(buf);
         buf=dataFile.readStringUntil(X);
         lcd.print(buf);
         lcd.print("V ");
         buf=dataFile.readStringUntil(X);
         lcd.setCursor(0,1);
         lcd.print(buf);
         lcd.print(" ");
         buf=dataFile.readStringUntil(lf);
         lcd.print(buf);               
         }           
         if (digitalRead(2) == LOW) {goto ende2;}
        }
        ende2:
       dataFile.close();
       y=true;
      }
      // if the file isn't open, pop up an error:
      
      else 
      {
       if (Debug == true) Serial.println("error opening log File");  
      }
      Werte=   "Ende weiter > A ";
      PRINT(0);
      delay(1000);
      y=true;
      break;
    }

     if (digitalRead(6) == LOW and y==true)  // loeschen Taste 3
      {          
       y=false;
       delay(250);
       Werte="Bestaetigen = T4";
       PRINT(0);
       y=true;  
       
       while (digitalRead(7) != LOW and y==true)  // Bestätigung mit Taste 4
       {
        y=false;
        delay(250);
        y=true;
        if (digitalRead(2) == LOW and y==true) 
         {
          y=false;
          delay(250);
          y=true;
          goto ende;
         }
       }  // bestätigen abwarten, Abbruch mit Auswahltaste
 
       SD.remove("datalog.txt");
       delay(400);
       File dataFile = SD.open("datalog.txt", FILE_WRITE);
       dataFile.close();
       Werte="   geloescht    ";
       if (Debug == true) Serial.println("Geloescht!");
       ende:
       PRINT(0);
       delay(1000);
      }   
   }
}


void loop() // Main
{
  digitalWrite(8, HIGH);  // AC-Sensor ausschalten um I2C Bus Last zu reduzieren
  digitalWrite(9, LOW); // Alarm Ausgang ausschalten
  y=true;
  ALARM=false;
  if (InterruptDone == 1) // Eingabe: ist die Auswahltaste betätigt?
  {  
   switch(FunktionCounter)
   {   
   case 1:
   UHR();
   break;
   
   case 2:
   UHRSET();
   break;
   
   case 3:
   TRIGGER();
   break;
     
   case 4: 
   RUNSTOP();
   break;
   
   case 5:
   SPEICHER();
   break;

   case 6:
   Messung();
   break;

   case 7:
   Logger();
   break;
   }
  }
 }
